#include <stdio.h>
#include <time.h>

#include "dp_blacklist.h"
#include "dp_helpers.h"
#include "dp_nfqueue.h"

static dp_blacklist *blacklists = NULL;
static time_t dp_cleanuptime = 0;

void dp_blacklist_add(dp_conf *conf, unsigned char *remoteid, int len) {
	dp_blacklist *bl;

	// alrady exists?
	HASH_FIND(hh, blacklists, remoteid, len, bl);

	if ( conf->debug ) printf("BL: add item\n");

	// found an entry, push the expiration further
	if ( bl ) {
		if ( conf->debug ) printf("BL: item found -> pushing further\n");
		bl->expire = time(NULL) + conf->bltime;
	}
	// not found, create a new one
	else {
		if ( conf->debug ) printf("BL: item not found, new entry in BL\n");
		dp_log(remoteid, len, "blacklisting started");
		bl = malloc(sizeof(dp_blacklist));
		memcpy(bl->remoteid, remoteid, len);
		bl->len = len;
		bl->expire = time(NULL) + conf->bltime;
		HASH_ADD(hh, blacklists, remoteid, len, bl);
	}
}

void dp_blacklist_cleanup(dp_conf *conf) {
	dp_blacklist *bl, *bltmp;

	// blacklist cleanup check every 1 sec
	if ( dp_cleanuptime < time(NULL) ) {
		if ( conf->debug ) printf("cleanup BL\n");
		dp_cleanuptime = time(NULL);
		HASH_ITER(hh, blacklists, bl, bltmp) {
			if ( bl->expire < time(NULL) ) {
				dp_log(
					bl->remoteid, bl->len, 
					"blacklisting ended");
				HASH_DEL(blacklists, bl);
				free(bl);
			}
		}
	}
}

int dp_blacklist_check(dp_conf *conf, unsigned char *remoteid, int len) {
	dp_blacklist *bl;

	HASH_FIND(hh, blacklists, remoteid, len, bl);

	if ( conf->debug ) printf("BL Check\n");

	if ( bl ) {

		if ( bl->expire > time(NULL) ) {
			if ( conf->debug ) printf("blacklisted!\n");
			return NF_DROP;
		}
	}
	return NF_ACCEPT;
}
