#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <syslog.h>

#include "dp_helpers.h"

static int dp_syslog = 0;

// syslog function
void dp_log(unsigned char *remoteid, int remoteidlen, char *fmt, ...) {
	va_list argList;
	char buf[1000];
	int offset = remoteidlen*2;
	int i;

	if ( ! dp_syslog ) {
		dp_syslog = 1;
		openlog("dhcp_protect", LOG_PID, LOG_DAEMON);
	}

	for(i=0; i<remoteidlen; i++) {
		sprintf(buf+(i*2), "%02x", remoteid[i]);
	}
	buf[offset]=':'; offset++;
	buf[offset]=' '; offset++;

	va_start(argList, fmt);
	vsnprintf(buf+offset, sizeof(buf)-offset-1, fmt, argList);
	va_end(argList);

	syslog(LOG_DAEMON|LOG_INFO, "%s", buf);
}

// display usage
void dp_usage(char *prog) {
	fprintf(stderr,"Usage: %s <configuration file>\n",prog);
}


// load configuration file
dp_conf *dp_load_config(dp_conf *conf, char *file) {
	FILE *fh;
	char *line = NULL;
	size_t len = 0;
	int error = 0;

	printf("Loading configuration %s\n",file);
	

	if ( ( fh = fopen(file,"r") ) == NULL ) {
		fprintf(stderr,"Failed to open configuration file '%s': %s\n", file, strerror(errno));
		return NULL;
	}

	while (getline(&line, &len, fh) != -1) {
		char *name, *value;

		name  = strtok(line, "=\r\n ");
		value = strtok(NULL,"=\r\n ");

		if ( name == NULL || value == NULL || name[0] == '#' )
			continue;

		if ( strcmp(name,"max_pkt_per_interval")==0 )
			conf->pktint = atoi(value);
		else if ( strcmp(name,"interval")==0 )
			conf->interval = atoi(value);
		else if ( strcmp(name, "debug")==0 )
			conf->debug = atoi(value) ? 1 : 0;
		else if ( strcmp(name, "blacklist_time")==0 )
			conf->bltime = atoi(value);
		else if ( strcmp(name, "queue")==0 )
			conf->queue = atoi(value);
		else if ( strcmp(name, "dryrun")==0 )
			conf->dryrun = atoi(value) ? 1 : 0;
		else
			fprintf(stderr,"unknown directive '%s', ignored\n", name);

		free(line);
	}
	fclose(fh);

	if ( conf->pktint < 1 || conf->pktint > 1000 ) {
		fprintf(stderr,"max_pkt_per_interval value invalid (min 1, max 1000)\n");
		error=1;
	}
	if ( conf->interval < 5 || conf->interval > 900 ) {
		fprintf(stderr,"interval value invalid (min 5, max 900)\n");
		error=1;
	}
	if ( conf->debug < 0 || conf->debug > 1 ) {
		fprintf(stderr,"debug value invalid (0 or 1)\n");
		error=1;
	}
	if ( conf->bltime < 10 || conf->bltime > 900 ) {
		fprintf(stderr,"blacklist_time value invalid (min 10, max 900)\n");
		error=1;
	}
	if ( conf->queue < 0 ) {
		fprintf(stderr,"queue must be a positive integer\n");
		error=1;
	}
	if ( conf->dryrun < 0 || conf->dryrun > 1 ) {
		fprintf(stderr, "dryrun value invalid (0 or 1)\n");
		error=1;
	}

	if ( error )
		return NULL;

	printf("Configuration:\n");
	printf("\t%-20s = %4s\n", "dryrun", conf->dryrun ? "Yes" : "No");
	printf("\t%-20s = %4s\n", "debug", conf->debug ? "Yes" : "No" );
	printf("\t%-20s = %4is\n", "interval", conf->interval);
	printf("\t%-20s = %4i\n", "max_pkt_per_interval", conf->pktint);
	printf("\t%-20s = %4is\n", "blacklist_time", conf->bltime);
	printf("\t%-20s = %4i\n", "queue", conf->queue);


	return conf;
}
